/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
"use strict";
var types_1 = require("./base/types");
// ------ internal util
var _util;
(function (_util) {
    _util.DI_TARGET = "$di$target";
    _util.DI_CONSTRUCTOR_DEPENDENCIES = "$di$constructorDependencies";
    _util.DI_PROPERTY_DEPENDENCIES = "$di$propertyDependencies";
    function getConstructorServiceDependencies(ctor) {
        return ctor[_util.DI_CONSTRUCTOR_DEPENDENCIES] || [];
    }
    _util.getConstructorServiceDependencies = getConstructorServiceDependencies;
    function getPropertyServiceDependencies(ctor) {
        return ctor[_util.DI_PROPERTY_DEPENDENCIES] || [];
    }
    _util.getPropertyServiceDependencies = getPropertyServiceDependencies;
})(_util = exports._util || (exports._util = {}));
exports.IInstantiationService = createDecorator("instantiationService");
function ensureDependencyMetadata(target) {
    if (target[_util.DI_TARGET] !== target) {
        target[_util.DI_TARGET] = target;
        target[_util.DI_CONSTRUCTOR_DEPENDENCIES] = [];
        target[_util.DI_PROPERTY_DEPENDENCIES] = [];
    }
}
function storePropertyDependency(id, target, key, optional) {
    ensureDependencyMetadata(target);
    target[_util.DI_PROPERTY_DEPENDENCIES].push({ id: id, key: key, optional: optional });
}
function storeConstructorDependency(id, target, index, optional) {
    ensureDependencyMetadata(target);
    target[_util.DI_CONSTRUCTOR_DEPENDENCIES].push({ id: id, index: index, optional: optional });
}
/**
 * A *only* valid way to create a {{ServiceIdentifier}}.
 */
function createDecorator(serviceId) {
    var id = function (target, key, indexOrPropDescriptor) {
        if (arguments.length !== 3) {
            throw new Error("@IServiceName-decorator can only be used to decorate a parameter or property");
        }
        if (types_1.isUndefined(indexOrPropDescriptor) || types_1.isObject(indexOrPropDescriptor)) {
            storePropertyDependency(id, target.constructor, key, false);
        }
        else {
            storeConstructorDependency(id, target, indexOrPropDescriptor, false);
        }
    };
    id.toString = function () { return serviceId; };
    return id;
}
exports.createDecorator = createDecorator;
/**
 * Mark a service dependency as optional.
 */
function optional(serviceIdentifier) {
    return function (target, key, indexOrPropDescriptor) {
        if (arguments.length !== 3) {
            throw new Error("@optional-decorator can only be used to decorate a parameter or property");
        }
        if (types_1.isUndefined(indexOrPropDescriptor) || types_1.isObject(indexOrPropDescriptor)) {
            storePropertyDependency(serviceIdentifier, target.constructor, key, true);
        }
        else {
            storeConstructorDependency(serviceIdentifier, target, indexOrPropDescriptor, true);
        }
    };
}
exports.optional = optional;
